#include "Model.hpp"


cx_mat model::vitesse_zz(double kz) // Velocity on the z axis
{
    cx_mat v(2,2, fill::zeros);
    v(0,0) = 2.*std::sin(kz);
    v(1,1) = -2.*std::sin(kz);
    return v;
}

cx_mat model::GreenR(double w, double kx, double ky, double kz){
    cx_mat H = this->H(kx,ky, kz);
    cx_mat Id(2,2, fill::eye);
    cx_dl I(0,1.);
    Id *= w+mu+eta*I;    
    Id -= H;
    return inv(Id); 
}
cx_mat model::GreenRInt(double w, double kx, double ky, double kz, cx_dl S){
    cx_mat H = this->HInt(kx,ky, kz, S);
    cx_mat Id(2,2, fill::eye);
    cx_dl I(0,1.);
    Id *= w+mu+eta*I;    
    Id -= H;
    return inv(Id); 
}


cx_mat model::A(double w,  double kx, double ky, double kz){
    cx_dl I(0,1.);
    cx_mat GR = GreenR(w,kx, ky, kz);
    return (GR.t()-GR)/I;
}
cx_mat model::AInt(double w,  double kx, double ky, double kz, cx_dl S){
    cx_dl I(0,1.);
    cx_mat GR = GreenRInt(w,kx, ky, kz, S);
    return (GR.t()-GR)/I;
}

cx_mat model::H(double kx, double ky, double kz) // Free Hamiltonian
{
    cx_mat H(2,2, fill::zeros);
    cx_dl I(0.0,1.0);
    H(0,0) = -2*(std::cos(kx)+std::cos(ky)+std::cos(kz));
    H(0,1) = 2.*std::sin(kx)-2.0*I*std::sin(ky);
    H(1,0) = 2.*std::sin(kx)+2.0*I*std::sin(ky);
    H(1,1) = 2*(std::cos(kx)+std::cos(ky)+std::cos(kz));
    return H;
}
cx_mat model::HInt(double kx, double ky, double kz, cx_dl S) 
{
    cx_mat Hk = H(kx,ky,kz);
    Hk(0,0) += S;
    Hk(1,1) += S;
    return Hk;
}
