
#ifndef Model_hpp
#define Model_hpp

#define _USE_MATH_DEFINES
#define ARMA_DONT_USE_WRAPPER

#include <iostream>
#include <iomanip>
#include <fstream>
#include <cmath>
#include <complex>
#include <vector>
#include "config.hpp"
#include <armadillo>

using namespace arma;

typedef std::complex<double> cx_dl;


class model{
    public:
    model(parameters param){
        this->beta = param.beta;
        this->Interacting = false;
        this->mu = param.mu;
        this ->eta = param.eta;
        this -> Interacting = param.Int;
    }
    cx_mat vitesse_zz(double kz);
    cx_mat GreenR(double w,  double kx, double ky, double kz);
    cx_mat GreenRInt(double w,  double kx, double ky, double kz, cx_dl);
    cx_mat GreenA(double w,  double kx, double ky, double kz);
    cx_mat A(double w,  double kx, double ky, double kz);
    cx_mat AInt(double w,  double kx, double ky, double kz, cx_dl S);
    cx_mat H( double kx, double ky, double kz);
    cx_mat HInt(double kx, double ky, double kz, cx_dl S);

    double get_beta(){
        return beta;
    }
    bool is_interacting(){return Interacting;}


    void print(){
        if(Interacting==false){
        std::cout<<"Model WSM without Interaction. beta="<< beta<<". mu = "<<mu<<"."<<std::endl;
        }
    }



    private:

    double beta;
    double mu;
    double eta;
    bool Interacting;

};




#endif

