# condRACpp

condRACpp is a c++ program that computes the optical conductivity of correlated Weyl
semi-metal. Using a lattice model which breaks time reversal symmetry (see.  [here](https://arxiv.org/abs/1807.07508)), the code can be adapted to different model
if you are brave enough.

## Requierements

The program is written in C++ and uses MPI parallelization in order to compute the optical
conductivity. It requires the following libraries:

1. **mpic++** as Open MPI C++ wrapper compiler. Tested on ubuntu using g++ 7.3.0, Apple LLVM version 10.0.0 and icpc version 16.0.4.

2. **Armadillo** as a C++ library for linear algebra. Tested with armadillo 9.100.6.

3. **LAPACK** and **BLAS** in order to speed up the algebra. Tested with LAPACK 3.8.0, mkl 11.3.4.258 and framework Accelerate (in mac os).

4. **Boost** library and more specificaly boost-MPI. Tested with Boost 1.60.0.

This can all be installed on Ubuntu 16.04 and 18.04 by the following command: </br>

```bash
sudo apt-get install build-essential libarmadillo-dev libboost-all-dev libopenmpi-dev liblapack-dev
```

## Installation

In order to compile the program, you have to change the makefile according the system you want to install the condRACpp on. Installation on Mac os X, ubuntu and mammouth is provided. After the change, just run the command:

```bash
make
```

## Usage

condRACpp needs a *param.txt* file in order to work. Exemple of this file could be found in the exemple folders.

The arguments in the parameters file are:

* Nw : The number of points for the frequency integral. The boundaries of this integral are defined depending on the frequency you want to compute the conductivity.
* beta : The inverse of temperature.
* Nk : The number of points in each direction of the integral on momenta.
* condRACpp computes the optical conductivity for different frequencies. The three following parameters are used to compute the frequency grid:
 * emin : First frequency.
 * emax : Last frequency.
 * Ne : Number of frequencies in the grid.
* eta : Lorentzian broadening.
* mu : Chemical potential.
* Interaction : F if you want the non-interacting conductivity, T if you want the interacting conductivity. A *Region1_SelfEnRA.dat* file must be provided.

If you want to run the program, just run the command:
```bash
mpirun -n N ./CondPara
```
Where N is the number of thread you want to use in your calculation.

**Please check the exemples provided in the *exemple* directory.**

## Modifications in order to adapt condRACpp to different lattice system

In condRACpp, the model is defined in the files Model.cpp and Model.hpp. Please change the functions *vitesse_zz* and *H* by the velocity and the Hamiltonian of your model.


## License

condRACpp was coded by [Shaheen Acheche](mailto:shaheen.acheche@usherbrooke.ca) for the resarch group of Professor Tremblay.

 condRACpp is under [MIT](https://choosealicense.com/licenses/mit/) licence.
