#include <boost/mpi/environment.hpp>
#include <boost/mpi/communicator.hpp>
#include <boost/serialization/vector.hpp>
#include <boost/serialization/string.hpp>
#include <boost/mpi.hpp>
#include <iostream>
#include "config.hpp"
#include "reading.hpp"
#include "Model.hpp"
#include "tools.hpp"
#include <assert.h>
#include <armadillo>
#include "Sigma.hpp"

using namespace arma;

namespace mpi = boost::mpi;



int main()
{
    mpi::environment env;
    mpi::communicator world;
    const parameters p = get_params();
    d_vector kgrid = linspace(0, 2.*M_PI, p.Nk);
    d_vector kz = linspace(0,2*M_PI, p.Nk);
    d_vector wgrid;
    double wmin;
    double wmax;
    d_vector e_grid = linspace(p.emin, p.emax, p.Ne, true);
    d_vector resultat(p.Nw);
    double integration;
    model mod(p);
    mod.print();
    std::ofstream txt;
    std::setprecision(18);
    d_vector w_f_Self;
    cd_vector Sigma_vec;
    Sigma Self;
    txt.open("test.dat");
    std::cout<<world.size()<<std::endl;

    if(mod.is_interacting()){
        std::cout<<"Getting real Frequency and Self"<<std::endl;
        w_f_Self = reading::read_real_freq();
        Sigma_vec = reading::read_Self_real();
        Self.mod_values(w_f_Self, Sigma_vec);
    }
    
    for(size_t i=0;i<p.Ne;i++){
        std::cout<<"Frequency w="<<e_grid[i]<<std::endl;
        std::cout<<"computation of boundaries"<<std::endl;
        get_boundary(mod.get_beta(),e_grid[i],wmin, wmax, 5.5);
        std::cout<<"New boundaries found. Wmin = "<<wmin<<". Wmax = "<<wmax<<std::endl;
        wgrid = linspace(wmin, wmax, p.Nw);
// ########################################################### MASTER #######################################################################################
        for(size_t ss = 0;ss<wgrid.size();++ss){
            if(world.rank()==0){
                int stepNk = p.Nk/(world.size());
                double step = 2.*M_PI/world.size();
                double recu{0.};
                d_vector test;

                for(size_t i =1; i<world.size();i++){
                world.send(i,0, linspace((i-1)*step, i*step,stepNk));
                }
                test = linspace(step*(world.size()-1), 2*M_PI, stepNk + p.Nk%stepNk);
                if(mod.is_interacting()){
                    integration = Trace_integrandeInt(mod, wgrid[ss], e_grid[i],Self.at(wgrid[ss]),Self.at(wgrid[ss]+e_grid[i]) , kgrid, test)/(1.*p.Nk);                
                }else{
                integration = Trace_integrande(mod, wgrid[ss], e_grid[i],kgrid, test)/(1.*p.Nk);
                }
                for(size_t j=1;j<world.size();j++){
                    world.recv(j,1, recu);
                    integration += recu;
                }
                if(e_grid[i]==0){
                    resultat[ss] = integration*dnFdw(mod.get_beta(),wgrid[ss]);
                }else{
                    resultat[ss] = integration*diff_nF(mod.get_beta(),wgrid[ss],e_grid[i]);
                }       
                }

//######################################################## SLAVE ############################################################################################
            if(world.rank()!=0){
                d_vector grid_rec;
                world.recv(0,0,grid_rec);
                if(mod.is_interacting()){
                    integration = Trace_integrandeInt(mod, wgrid[ss], e_grid[i],Self.at(wgrid[ss]),Self.at(wgrid[ss]+e_grid[i]) ,kgrid, grid_rec)/(1.*p.Nk);                
                }else{
                integration = Trace_integrande(mod, wgrid[ss], e_grid[i],kgrid, grid_rec)/(1.*p.Nk);
                }
                world.send(0,1,integration);
                //std::cout<<"Process #"<<world.rank()<<" done!"<<std::endl;
            }
        }
        if(world.rank()==0){
            txt<<e_grid[i]<<"    "<<simps(resultat,wgrid)<<std::endl;
        }
    }
    txt.close();
    //*/
    
    std::cout<<"Done"<<std::endl;
  return 0;
}
