
#include "tools.hpp"



d_vector linspace(double min, double max, int nbr, bool endpoint){
    d_vector d;
    if(nbr==1){
        d.push_back(min);
        return d;
    }
    if(min==max){
        d.push_back(max);
        return d;
    }
    d.reserve(nbr);
    if(endpoint==false){
    for(int i=0; i<nbr; i++){
        d.push_back(min +i*(max-min)/double(nbr));
    }
    }else{
    for(int i=0; i<nbr; i++){
        d.push_back(min +i*(max-min)/double(nbr-1));
    }   
    }
    return d;
}

double dnFdw(const double beta,const double w){
    if(abs(beta*w)>10){
        return 0.;
    }
    return beta/(4*cosh(0.5*beta*w)*cosh(0.5*beta*w));
}
double nF(const double beta, const double w){
    if(beta*w>10){
        return 0;
    }
    if(beta*w<-10){
        return 1.0;
    }
    return 1/(1+std::exp(beta*w));
}
double diff_nF(const double beta, const double w, const double eps){
    return (nF(beta, w)-nF(beta, w+eps))/eps;
}
double simps(d_vector y, d_vector x){
    int N = y.size();
    double h = (x.back()-x[0])/N;
    double res(0.);
    double inter1(0.);
    double inter2(0.);
    for(unsigned int i=1;i<N/2-1;i++){
        inter1+=y[2*i];
    }
    for(unsigned int i=1;i<N/2;i++){
        inter2+=y[2*(i-1)];
    }
    res = y[0]+2*inter1+4*inter2+y.back();
    return res*h/3.;
}


double Trace_integrande(model mod, const double w,const double eps, d_vector kgrid, d_vector kz){
    cx_mat Integrande(2,2);
    cx_dl res{0.};
    for(size_t i = 0; i< kgrid.size();i++){
        for(size_t j=0; j< kgrid.size();j++){
            for(size_t k=0;k<kz.size();k++){
                Integrande = mod.vitesse_zz(kz[k])*mod.A(w, kgrid[i], kgrid[j], kz[k])*mod.vitesse_zz(kz[k])*mod.A(w+eps, kgrid[i], kgrid[j], kz[k]);
                res += trace(Integrande);
            }
        }
    }
    assert(res.imag()<1e-10);

    return res.real()/(4*M_PI*kgrid.size()*kgrid.size());
}
double Trace_integrandeInt(model mod, const double w,const double eps, cx_dl S1, cx_dl S2, d_vector kgrid, d_vector kz){
    cx_mat Integrande(2,2);
    cx_dl res{0.};
    for(size_t i = 0; i< kgrid.size();i++){
        for(size_t j=0; j< kgrid.size();j++){
            for(size_t k=0;k<kz.size();k++){
                Integrande = mod.vitesse_zz(kz[k])*mod.AInt(w, kgrid[i], kgrid[j], kz[k],S1)*mod.vitesse_zz(kz[k])*mod.AInt(w+eps, kgrid[i], kgrid[j], kz[k], S2);
                res += trace(Integrande);
            }
        }
    }
    assert(res.imag()<1e-10);

    return res.real()/(4*M_PI*kgrid.size()*kgrid.size());
}


double Trace_integrande(model mod, const double w,const double eps, d_vector kgrid){
    cx_mat Integrande(2,2);
    cx_dl res{0.};
    for(size_t i = 0; i< kgrid.size();i++){
        for(size_t j=0; j< kgrid.size();j++){
            for(size_t k=0;k<kgrid.size();k++){
                Integrande = mod.vitesse_zz(kgrid[k])*mod.A(w, kgrid[i], kgrid[j], kgrid[k])*mod.vitesse_zz(kgrid[k])*mod.A(w+eps, kgrid[i], kgrid[j], kgrid[k]);
                res += trace(Integrande);
            }
        }
    }
    assert(res.imag()<1e-10);

    return res.real()/(4*M_PI*kgrid.size()*kgrid.size()*kgrid.size());
}

d_vector sigmaDC(model mod,d_vector wgrid,d_vector kgrid){
    d_vector sigma_int(wgrid.size());
    for(size_t i =0;i<wgrid.size();i++){
        sigma_int[i] = Trace_integrande(mod, wgrid[i],0, kgrid)*dnFdw(mod.get_beta(), wgrid[i]);
    }
    return sigma_int;
}

d_vector sigmaAC(model mod,double eps, d_vector wgrid,d_vector kgrid){
    d_vector sigma_int(wgrid.size());
    for(size_t i =0;i<wgrid.size();i++){
        sigma_int[i] = Trace_integrande(mod, wgrid[i],eps, kgrid)*diff_nF(mod.get_beta(), wgrid[i],eps);
    }
    return sigma_int;
}


void get_boundary(const double beta, const double eps, double& wmin, double& wmax, double coeff){
    if(eps>0){
        wmin = -eps - coeff/beta;
        wmax = coeff/beta;
    }else{
        wmin = -coeff/beta;
        wmax = -eps + coeff/beta;
    }
}